/*
 * Functions for elementary operations for
 * simplicial complex computations.
 *
 * by M.Hachimori, May 2000.
 */

#include <stdio.h>
#include <stdlib.h>
#include "s_complex.h"



/*
 * Function to enumerate all faces from
 * a list of facets.
 * The input list of facets are assumed to be sorted.
 * The output list is also sorted.
 */
Simplices *facets2faces(faces)
  Simplices *faces;
{
  Simplices *initial, *tail, *current, *output, *return_address, *tmp, *tmp2;
  int i, j, d, num_newface;

  if (faces == NULL)
    return NULL;

  output = (Simplices*)malloc(sizeof(Simplices));
  return_address = output;
  output->prev = NULL;
  output->next = NULL;
  initial = output; /* 'initial' be the first and 'tail' be the last data of new d-dim faces.*/
  d = faces->vert[0];

  current = faces;    /* 'current' is a pointer sweeping the input data. */

  /* copy the largest dimension part */
  while(current != NULL && current->vert[0] == d){
    output->next = (Simplices*)malloc(sizeof(Simplices));
    output->next->prev = output;
    output = output->next;
    for (i = 0; i <= current->vert[0]+1; i++)
      output->vert[i] = current->vert[i];
    output->next = NULL;
    current = current->next;
    }
  tail = output;
  d--;
  initial = initial->next; 

  while(output->vert[0] >= 1){
    num_newface = 0;
    /* copy 1-dim-lower faces to output.*/
    while (current!= NULL && current->vert[0] > d)
      current = current->next;
    while (current!= NULL && current->vert[0] == d){
      output->next = (Simplices*)malloc(sizeof(Simplices));
      for (i = 0; i <= current->vert[0]+1; i++)
        output->next->vert[i] = current->vert[i];
      output->next->prev = output;
      output = output->next;
      output->next = NULL;
      num_newface ++;
      current = current->next;}

  
    /* compute codim 1 faces of faces between `initial' and `tail'
       and add them to output. */
    while(initial != NULL && initial->vert[0] == d+1){
      for (i = 1; i <= initial->vert[0]+1; i ++){
        output->next = (Simplices*)malloc(sizeof(Simplices));
        output->next->prev = output;
        output = output->next;
        output->next = NULL;
        output->vert[0] = initial->vert[0] - 1;
        for (j = 1; j <= initial->vert[0]+1; j++) /* list all codim 1 faces */
          if (j < i) output->vert[j] = initial->vert[j];
          else if (j > i) output->vert[j-1] = initial->vert[j];
        num_newface ++;}
      initial = initial->next;
      }

    /* sort the faces and remove multiplicity before going to the next step*/
    initial = tail->next;
    msort(initial, num_newface);
    tmp = initial;
    while(tmp->next != NULL){
      if (cmp_faces(tmp->vert, (tmp->next)->vert) == 0){
        tmp2 = (tmp->next)->next;
        free(tmp->next);
        tmp->next = tmp2;
        if(tmp->next != NULL)
          tmp->next->prev = tmp;
        }
      else
        tmp = tmp->next;}
    tail = tmp;
    output = tmp;

    d --;
    }

  return_address = return_address->next;
  free(return_address->prev);
  return_address->prev = NULL;
  return return_address;
}


/*
 * Function to calculate a set of facets
 * from a set of faces.
 */
Simplices *faces2facets(faces)
  Simplices *faces;
{
  Simplices *return_address, *head, *tail, *current, *output, *tmp;
  int i, j, d, mark;

  if (faces == NULL) return NULL;


  d = faces->vert[0];
  output = (Simplices*)malloc(sizeof(Simplices));
  output->prev = NULL;
  output->next = NULL;
  return_address = output;

  current = faces;
  head = current;
  /* All the largest dimensional faces are copied to output. */
  while(current!= NULL && current->vert[0] == d){
    output->next = (Simplices*)malloc(sizeof(Simplices));
    output->next->prev = output;
    output = output->next;
    output->next = NULL;
    for (i = 0; i <= current->vert[0]+1; i ++)
      output->vert[i] = current->vert[i];
    current = current->next;
    }
  d --;

  /* For the smaller dimensional faces, check whether they are
     contained in a larger face or not. */
  while(d >= 0){
    while(current != NULL && current->vert[0] == d) {
      tmp = head;
      mark = 0;
      while(tmp != NULL && tmp->vert[0] > d) {
        if (face_contained(current->vert, tmp->vert) == 1){
          mark = 1;
          break;}
        tmp = tmp->next;
        }
      if (mark == 0){
        output->next = (Simplices*)malloc(sizeof(Simplices));
        output->next->prev = output;
        output = output->next;
        output->next = NULL;
        for (i = 0; i <= current->vert[0]+1; i ++)
          output->vert[i] = current->vert[i];
        }
      current = current->next;
      }

    d --;
  }

  /* remove the first extra data before returning. */
  return_address = return_address->next;
  free(return_address->prev);
  return_address->prev = NULL;
  return return_address;
}





/*
 * Function to calculate f-vector.
 *   Input is a set of faces
 *   Output is a vector whose i-th element is f_(i-1)
 *   (Note that the index is shifted by 1.
 *    If the input is NULL, then return f_vec[0] = 1,
 *    just show that there is only one (-1)-dim face.)
 */
void get_fvector(faces, f_vec)
  Simplices *faces;
  int f_vec[];
{
  int i;

  if (faces == NULL){
    f_vec[0] = 1;
    return;}

  f_vec[0] = 1;
  /* initially set f_vec[i] = 0 for all i>=1. */
  for (i = 1; i <= faces->vert[0]+1; i ++)
    f_vec[i] = 0;

  /* count the number of faces. */
  while(faces != NULL){
    f_vec[faces->vert[0]+1] ++;
    faces = faces->next;
    }

  return;
}


/*
 * Function to calculate h-vector from f-vector.
 */
void f_to_h(fvec, hvec, dim)
  int fvec[];
  int hvec[];
  int dim;
{
  int i, j;
  int tmp[MAXDIM+2], tmp2[MAXDIM+2];

  tmp[0] = fvec[0];
  for (i = 0; i <= dim+1; i ++)
    {
    tmp2[0] = 1;
    for (j = 1; j <= i; j ++)
      tmp2[j] = tmp[j] - tmp[j - 1];
    tmp2[i + 1] = fvec[i + 1];
    for (j = 0; j <= i + 1; j ++)
      tmp[j] = tmp2[j];
    }
  for(i = 0; i <= dim+1; i ++)
    hvec[i] = tmp[i];
  return;
}



